from datetime import datetime

from twisted.internet.defer import inlineCallbacks
from twisted.trial.unittest import TestCase

from txgsm.tests.base import TxGSMBaseTestCase, LogCatcher
from txgsm.service import TxGSMService
from txgsm.protocol import TxGSMProtocol

from mock import Mock


class TxGSMProtocolTestCase(TxGSMBaseTestCase):

    timeout = 1

    @inlineCallbacks
    def test_configure_modem(self):
        d = self.modem.configure_modem()
        yield self.assertExchange(['ATE0'], ['OK'])
        yield self.assertExchange(['AT+CMGF=0'], ['OK'])
        yield self.assertExchange(['AT+CMEE=1'], ['OK'])
        yield self.assertExchange(['AT+CSMS=1'], ['OK'])
        response = yield d
        self.assertEqual(response, [
            {'command': ['ATE0'], 'expect': 'OK', 'response': ['OK']},
            {'command': ['AT+CMGF=0'], 'expect': 'OK', 'response': ['OK']},
            {'command': ['AT+CMEE=1'], 'expect': 'OK', 'response': ['OK']},
            {'command': ['AT+CSMS=1'], 'expect': 'OK', 'response': ['OK']}
        ])

    @inlineCallbacks
    def test_send_sms(self):
        d = self.modem.send_sms('+27761234567', 'hello world')
        yield self.assertCommands(['AT+CMGS=23'])
        self.reply('> ', delimiter='')
        [pdu_payload] = yield self.wait_for_next_commands()
        self.reply('OK')
        response = yield d
        self.assertEqual(response, [
            {
                'command': ['AT+CMGS=23'],
                'expect': '> ',
                'response': ['> ']
            },
            {
                'command': ['0001000B917267214365F700000B'
                            'E8329BFD06DDDF723619\x1a'],
                'expect': 'OK',
                'response': ['OK']
            }
        ])

    @inlineCallbacks
    def test_send_multipart_sms(self):
        d = self.modem.send_sms('+27761234567', '1' * 180)
        yield self.assertCommands(['AT+CMGS=153'])
        self.reply('> ', delimiter='')
        [pdu_payload] = yield self.wait_for_next_commands()
        self.reply('OK')
        yield self.assertCommands(['AT+CMGS=43'])
        self.reply('> ', delimiter='')
        [pdu_payload] = yield self.wait_for_next_commands()
        self.reply('OK')
        response = yield d
        self.assertEqual(response, [
            {
                'command': ['AT+CMGS=153'],
                'expect': '> ',
                'response': ['> ']
            }, {
                'command': ['0041000B917267214365F70000A005000301020162B158'
                            '2C168BC562B1582C168BC562B1582C168BC562B1582C16'
                            '8BC562B1582C168BC562B1582C168BC562B1582C168BC5'
                            '62B1582C168BC562B1582C168BC562B1582C168BC562B1'
                            '582C168BC562B1582C168BC562B1582C168BC562B1582C'
                            '168BC562B1582C168BC562B1582C168BC562B1582C168B'
                            'C562B1582C168BC562B1582C168BC562\x1a'],
                'expect': 'OK',
                'response': ['OK']
            }, {
                'command': ['AT+CMGS=43'],
                'expect': '> ',
                'response': ['> ']
            }, {
                'command': ['0041000B917267214365F700002205000301020262B158'
                            '2C168BC562B1582C168BC562B1582C168BC562B118\x1a'],
                'expect': 'OK',
                'response': ['OK']
            }]
        )

    @inlineCallbacks
    def test_receive_sms(self):
        d = self.modem.list_received_messages(status=4)
        # Using PDU samples from
        # http://www.diafaan.com/sms-tutorials/gsm-modem-tutorial/at-cmgl-pdu-mode/
        yield self.assertExchange(
            input=['AT+CMGL=4'],
            output=[
                '+CMGL: 1,0,,39',
                ('07911326040011F5240B911326880736F400001110810173624016547' +
                 '47A0E4ACF41F4329E0E6A97E7F3F0B90C8A01'),
                '+CMGL: 2,0,,39',
                ('07911326040011F5240B911326880736F400001110810173234016547' +
                 '47A0E4ACF41F4329E0E6A97E7F3F0B90C9201'),
                'OK'
            ])
        result = yield d
        [sms1, sms2] = result
        self.assertEqual(sms1.data, {
            'csca': '+31624000115',
            'sr': None,
            'type': None,
            'date': datetime(2011, 1, 18, 9, 37, 26),
            'text': u'This is text message 1',
            'fmt': 0,
            'pid': 0,
            'dcs': 0,
            'number': '+31628870634',
        })
        self.assertEqual(sms2.data, {
            'csca': '+31624000115',
            'sr': None,
            'type': None,
            'date': datetime(2011, 1, 18, 9, 37, 32),
            'text': u'This is text message 2',
            'fmt': 0,
            'pid': 0,
            'dcs': 0,
            'number': '+31628870634',
        })

    @inlineCallbacks
    def test_ussd_session(self):
        d = self.modem.dial_ussd_code('*100#')
        yield self.assertExchange(
            input=['AT+CUSD=1,"*100#",15'],
            output=[
                'OK',
                ('+CUSD: 2,"Your balance is R48.70. Out of Airtime? '
                 'Dial *111# for Airtime Advance. T&Cs apply.",255')
            ])
        result = yield d
        response = result['response']
        self.assertEqual(response[0], 'OK')
        self.assertTrue(response[1].startswith('+CUSD: 2'))

    def test_dealing_with_unexpected_events(self):
        with LogCatcher() as catcher:
            self.reply('+FOO')
            [err_log] = catcher.logs
            self.assertTrue('Unsollicited response' in err_log['message'][0])
            self.assertTrue('+FOO' in err_log['message'][0])

    @inlineCallbacks
    def test_probe(self):
        d = self.modem.probe()
        yield self.assertExchange(['ATE0'], ['OK'])
        yield self.assertExchange(['AT+CIMI'], ['01234123412341234', 'OK'])
        yield self.assertExchange(['AT+CGMM'], ['Foo Bar Corp', 'OK'])
        response = yield d
        self.assertEqual(response, [
            {
                'command': ['ATE0'],
                'expect': 'OK',
                'response': ['OK']
            }, {
                'command': ['AT+CIMI'],
                'expect': 'OK',
                'response': ['01234123412341234', 'OK']
            }, {
                'command': ['AT+CGMM'],
                'expect': 'OK',
                'response': ['Foo Bar Corp', 'OK']
            }
        ])


class TxGSMServiceTestCase(TestCase):

    def setUp(self):
        self.mock_serial = Mock()
        self.service = TxGSMService('/dev/foo', bar='baz')
        self.service.serial_port_class = self.mock_serial

    @inlineCallbacks
    def test_start_service(self):
        d = self.service.onProtocol
        self.service.startService()
        protocol = yield d
        self.assertTrue(isinstance(protocol, TxGSMProtocol))
        self.assertTrue(self.mock_serial.called)
        [init_call] = self.mock_serial.call_args_list
        args, kwargs = init_call
        proto, device, reactor = args
        self.assertEqual(device, '/dev/foo')
        self.assertEqual(kwargs, {'bar': 'baz'})

    def test_stop_service(self):
        self.service.startService()
        self.service.port.loseConnection = Mock()
        self.service.stopService()
        self.assertTrue(self.service.port.loseConnection.called)



from twisted.trial.unittest import TestCase
from twisted.internet.defer import inlineCallbacks, Deferred
from twisted.internet import reactor
from twisted.test import proto_helpers
from twisted.python import log

from txgsm.protocol import TxGSMProtocol


class TxGSMBaseTestCase(TestCase):

    def setUp(self):
        self.modem = TxGSMProtocol()
        self.modem.verbose = True
        self.modem_transport = proto_helpers.StringTransport()
        self.modem.makeConnection(self.modem_transport)

    def reply(self, data, delimiter=None, modem=None):
        modem = modem or self.modem
        dl = delimiter or modem.delimiter
        modem.dataReceived(data + dl)

    def wait_for_next_commands(self, clear=True, modem=None, transport=None):

        modem = modem or self.modem
        transport = transport or self.modem_transport

        d = Deferred()

        def check_for_input():
            if not transport.value():
                reactor.callLater(0, check_for_input)
                return

            commands = transport.value().split(modem.delimiter)

            if clear:
                transport.clear()

            d.callback(filter(None, commands))

        check_for_input()
        return d

    @inlineCallbacks
    def assertCommands(self, commands, clear=True, modem=None, transport=None):
        received_commands = yield self.wait_for_next_commands(
            clear=clear, modem=modem, transport=transport)
        self.assertEqual(commands, received_commands)

    @inlineCallbacks
    def assertExchange(self, input, output, clear=True, modem=None,
                       transport=None):
        yield self.assertCommands(input, clear=clear, modem=modem,
                                  transport=transport)
        for reply in output:
            self.reply(reply, modem=modem)


# Shamelessly copyied from @Hodgestar's contribution to
# https://github.com/praekelt/vumi/
class LogCatcher(object):

    def __init__(self):
        self.logs = []

    def __enter__(self):
        log.theLogPublisher.addObserver(self.logs.append)
        return self

    def __exit__(self, *exc_info):
        log.theLogPublisher.removeObserver(self.logs.append)



# -*- test-case-name: txgsm.tests.test_utils -*-

from twisted.protocols.basic import LineReceiver

from os import linesep


def quote(s):
    return s.replace('"', '\"')


class Console(LineReceiver):
    delimiter = linesep

    def __init__(self, on_input, prefix=''):
        self.prefix = prefix
        self.on_input = on_input

    def prompt(self):
        self.transport.write("%s> " % (self.prefix,))

    def lineReceived(self, line):
        return self.on_input(line)


class USSDConsole(Console):

    NO_FURTHER_ACTION = 0
    FURTHER_ACTION = 1
    NETWORK_TERMINATED = 2
    OTHER_CLIENT_RESPONDED = 3
    OPERATION_NOT_SUPPORTED = 4
    NETWORK_TIMEOUT = 5

    def __init__(self, modem, on_exit):
        Console.__init__(self, self.on_input, prefix='USSD ')
        self.modem = modem
        self.on_exit = on_exit

    def dial(self, number):
        d = self.modem.dial_ussd_code(number)
        d.addCallback(self.handle_response)
        return d

    def parse_ussd_response(self, resp):
        for item in resp:
            if not item.startswith('+CUSD'):
                continue

            ussd_resp = item.lstrip('+CUSD: ')
            operation = ussd_resp[0]
            content = ussd_resp[3:-4]
            return int(operation), content

    def on_input(self, line):
        d = self.modem.send_command('AT+CUSD=1,"%s",15' % (quote(line),),
                                    expect="+CUSD")
        d.addCallback(self.handle_response)
        return d

    def handle_response(self, result):
        operation, content = self.parse_ussd_response(result['response'])
        self.sendLine(content)
        if operation == self.FURTHER_ACTION:
            return self.prompt()
        else:
            return self.on_exit(operation)



from twisted.test import proto_helpers

from txgsm.tests.base import TxGSMBaseTestCase
from txgsm.utils import USSDConsole

from mock import Mock


class USSDConsoleTestCase(TxGSMBaseTestCase):

    def setUp(self):
        super(USSDConsoleTestCase, self).setUp()
        self.exit = Mock()
        self.console = USSDConsole(self.modem, on_exit=self.exit)
        self.console_transport = proto_helpers.StringTransport()
        self.console.makeConnection(self.console_transport)

    def test_dial_single_screen_session(self):
        d = self.console.dial('*100#')
        self.assertExchange(
            input=['AT+CUSD=1,"*100#",15'],
            output=[
                'OK',
                '+CUSD: 2,"foo",25'
            ])
        self.assertEqual(self.console_transport.value(), 'foo\n')
        return d

    def test_dial_multiple_screen_session(self):
        d = self.console.dial('*100#')
        self.assertExchange(
            input=['AT+CUSD=1,"*100#",15'],
            output=[
                'OK',
                '+CUSD: 1,"what is your name?",25'
            ])
        self.assertEqual(self.console_transport.value(),
                         'what is your name?\n%s> ' % (self.console.prefix,))
        self.console_transport.clear()
        self.console.lineReceived('foo')
        self.assertExchange(
            input=['AT+CUSD=1,"foo",15'],
            output=[
                'OK',
                '+CUSD: 2,"thanks!",25'
            ])
        self.assertEqual(self.console_transport.value(), 'thanks!\n')
        self.assertTrue(self.exit.called)
        return d



from setuptools import setup


def listify(filename):
    return filter(None, open(filename, 'r').read().split('\n'))

install_requires = listify("requirements.txt")

with open('VERSION', 'r') as fp:
    version = fp.read().strip()

setup(
    name="txgsm",
    version=version,
    url='http://github.com/smn/txgsm',
    license='BSD',
    description="Utilities for talking to a GSM modem over USB via AT "
                "commands.",
    long_description=open('README.rst', 'r').read(),
    author='Simon de Haan',
    author_email='simon@praekeltfoundation.org',
    packages=[
        "txgsm",
        "twisted.plugins",
    ],
    package_data={
        'twisted.plugins': ['twisted/plugins/txgsm_plugin.py'],
    },
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Networking',
        'Framework :: Twisted',
    ],
)



# -*- test-case-name: txgsm.tests.test_protocol -*-
# -*- coding: utf-8 -*-
from twisted.internet import reactor
from twisted.protocols.basic import LineReceiver
from twisted.internet.defer import Deferred
from twisted.python import log

from .utils import quote

from messaging.sms import SmsSubmit, SmsDeliver


class TxGSMProtocol(LineReceiver):

    CTRL_Z = '\x1a'
    delimiter = '\r\n'
    verbose = False

    def __init__(self):
        # AT switches being purely line oriented and sometimes not
        # especially when sending multipart SMSs which has a '^> $' prompt
        # without a '\r\n' that we need to wait for.
        # As a result using lineReceived() does not always work.
        self.setRawMode()
        self.deferreds = []
        self.buffer = b''

    def log(self, msg):
        if self.verbose:
            log.msg(msg)

    def connectionMade(self):
        self.log('Connection made')

    def send_command(self, command, expect='OK', timeout=None):
        self.log('Sending: %r' % (command,))
        resp = Deferred()
        resp.addCallback(self.debug)
        if timeout:
            reactor.callLater(timeout, resp.cancel)
        self.deferreds.append((command, expect, resp))
        self.sendLine(command)
        return resp

    def debug(self, resp):
        self.log('Received: %r' % (resp,))
        return resp

    def next(self, command, expect='OK'):
        def handler(result):
            d = self.send_command(command, expect)
            d.addCallback(lambda r: result + [r])
            return d
        return handler

    def configure_modem(self):
        # Sensible defaults shamelessly copied from pygsm.
        d = Deferred()
        d.addCallback(self.next('ATE0'))  # Disable echo
        d.addCallback(self.next('AT+CMGF=0'))  # PDU mode
        d.addCallback(self.next('AT+CMEE=1'))  # More useful errors
        d.addCallback(self.next('AT+CSMS=1'))  # set SMS mode to phase 2+
        d.callback([])
        return d

    def send_sms(self, msisdn, text):
        sms = SmsSubmit(msisdn, text)
        # NOTE: The use of the Deferred here is a bit wonky
        #       I'm using it like this because it makes adding callbacks
        #       in a for-loop easier since we're potentially sending
        #       SMSs bigger than 160 chars.
        d = Deferred()
        for pdu in sms.to_pdu():
            d.addCallback(self.next(
                'AT+CMGS=%d' % (pdu.length,),
                expect='> '))
            d.addCallback(self.next('%s%s' % (pdu.pdu, self.CTRL_Z)))

        d.callback([])
        return d

    def dial_ussd_code(self, code):
        return self.send_command('AT+CUSD=1,"%s",15' % (quote(code),),
                                 expect='+CUSD')

    def list_received_messages(self, status=4):
        d = self.send_command('AT+CMGL=%i' % (status,))

        def parse_cmgl_response(result):
            response = result['response']
            # Lines alternative between the +CMGL response and the
            # actual PDU containing the SMS
            found = False
            messages = []
            for line in response:
                if line.startswith('+CMGL:'):
                    found = True
                elif found:
                    messages.append(SmsDeliver(line))
                    found = False

            return messages

        d.addCallback(parse_cmgl_response)
        return d

    def probe(self):
        """
        See if we're talking to something GSM-like and if so,
        try and get some useful information out of it.
        """
        d = Deferred()
        d.addCallback(self.next('ATE0'))
        d.addCallback(self.next('AT+CIMI'))
        d.addCallback(self.next('AT+CGMM'))
        reactor.callLater(0, d.callback, [])
        return d

    def rawDataReceived(self, data):
        self.buffer += data

        if not self.deferreds:
            log.err('Unsollicited response: %r' % (data,))
            return

        _, expect, _ = self.deferreds[0]

        if expect in self.buffer:
            command, expect, deferred = self.deferreds.pop(0)
            return_buffer, self.buffer = self.buffer, b''
            result = {
                'command': [command],
                'expect': expect,
                'response': filter(None, return_buffer.split(self.delimiter))
            }
            deferred.callback(result)



# -*- test-case-name: txgsm.tests.test_service -*-
import sys
from zope.interface import implementer

from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import Service
from twisted.application.service import IServiceMaker
from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
from twisted.internet import reactor, stdio
from twisted.internet.serialport import SerialPort
from twisted.python import log

from txgsm.protocol import TxGSMProtocol
from txgsm.utils import USSDConsole


class SendSMS(usage.Options):

    optParameters = [
        ["to-addr", None, None, "The address to send an SMS to."],
        ["message", None, None, "The message to send"],
    ]


class ListSMS(usage.Options):
    optParameters = [
        ['status', 's', 4, 'What messages to read (0: unread, 1: read, '
            '2: unsent, 3: sent, 4: all)'],
    ]


class USSDSession(usage.Options):

    optParameters = [
        ['code', None, None, "The USSD code to dial"],
    ]


class ProbeModem(usage.Options):
    pass


class Options(usage.Options):

    subCommands = [
        ['send-sms', None, SendSMS, "Send an SMS"],
        ['list-sms', None, ListSMS, "List SMSs on Modem"],
        ['ussd-session', None, USSDSession, 'Start a USSD session'],
        ['probe-modem', None, ProbeModem,
            'Probe the device to see if it is something modem-ish'],
    ]

    optFlags = [
        ["verbose", "v", "Log AT commands"],
    ]

    optParameters = [
        ["device", "d", None, "The device to connect to."],
        ["baudrate", "b", 9600, "Baud rate such as 9600 or 115200."],
        ["timeout", "t", None, "Set a read timeout value."],
    ]


class TxGSMService(Service):

    protocol = TxGSMProtocol
    serial_port_class = SerialPort

    def __init__(self, device, **conn_options):
        self.device = device
        self.conn_options = conn_options
        self.onProtocol = Deferred()
        self.onProtocol.addErrback(log.err)

    def getProtocol(self):
        return self.protocol()

    def startService(self):
        p = self.getProtocol()
        self.port = self.serial_port_class(p, self.device, reactor,
                                           **self.conn_options)
        self.onProtocol.callback(p)

    def stopService(self):
        self.port.loseConnection()


@implementer(IServiceMaker, IPlugin)
class TxGSMServiceMaker(object):
    tapname = 'txgsm'
    description = ("Utilities for talking to a GSM modem over USB via AT "
                   "commands.")
    options = Options

    def makeService(self, options):
        device = options['device']
        conn_options = {
            'baudrate': int(options['baudrate']),
            'timeout': None
        }
        if options['timeout'] is not None:
            conn_options['timeout'] = int(options['timeout'])

        service = TxGSMService(device, **conn_options)
        service.onProtocol.addCallback(self.set_verbosity, options)

        dispatch = {
            'send-sms': self.send_sms,
            'list-sms': self.list_sms,
            'ussd-session': self.ussd_session,
            'probe-modem': self.probe_modem,
        }

        callback = dispatch.get(options.subCommand)
        if callback:
            service.onProtocol.addCallback(callback, options)
            return service
        else:
            sys.exit(str(options))

    def set_verbosity(self, modem, options):
        modem.verbose = options['verbose']
        return modem

    @inlineCallbacks
    def send_sms(self, modem, options):
        cmd_options = options.subOptions
        yield modem.configure_modem()
        result = yield modem.send_sms(cmd_options['to-addr'],
                                      cmd_options['message'])
        returnValue(self.shutdown(result))

    @inlineCallbacks
    def list_sms(self, modem, options):
        cmd_options = options.subOptions
        yield modem.configure_modem()
        messages = yield modem.list_received_messages(
            int(cmd_options['status']))
        for message in messages:
            log.msg(message.data)
        returnValue(self.shutdown())

    @inlineCallbacks
    def ussd_session(self, modem, options):
        log.msg('Connecting to modem.')
        cmd_options = options.subOptions
        yield modem.configure_modem()
        log.msg('Connected, starting console for: %s' % (cmd_options['code'],))
        console = USSDConsole(modem, on_exit=self.shutdown)
        sio = stdio.StandardIO(console)
        log.msg('Dialing: %s' % (cmd_options['code'],))
        yield console.dial(cmd_options['code'])
        returnValue(sio)

    @inlineCallbacks
    def probe_modem(self, modem, options):
        result = yield modem.probe()
        [_, imsi_result, manufacturer_result] = result
        imsi, ok = imsi_result['response']
        manufacturer, ok = manufacturer_result['response']
        log.msg('Manufacturer: %s' % (manufacturer,))
        log.msg('IMSI: %s' % (imsi,))
        self.shutdown()

    def shutdown(self, resp=None):
        reactor.callLater(0, reactor.stop)



from twisted.internet.defer import inlineCallbacks

from txgsm.tests.base import TxGSMBaseTestCase, LogCatcher
from txgsm.service import TxGSMServiceMaker, TxGSMService, Options
from txgsm.utils import USSDConsole

from mock import Mock


class TxGSMServiceTestCase(TxGSMBaseTestCase):

    def setUp(self):
        super(TxGSMServiceTestCase, self).setUp()
        TxGSMService.serial_port_class = Mock()
        self.patch(TxGSMService, 'getProtocol', self.patch_get_protocol)
        self.patch(TxGSMServiceMaker, 'shutdown', self.patch_shutdown)

    def patch_get_protocol(self):
        #  Protocol created by TxGSMBaseTestCase
        return self.modem

    def patch_shutdown(self, result=None):
        # noop, pass along result instead of shutting down
        return result

    @inlineCallbacks
    def assert_configure_modem(self):
        yield self.assertExchange(['ATE0'], ['OK'])
        yield self.assertExchange(['AT+CMGF=0'], ['OK'])
        yield self.assertExchange(['AT+CMEE=1'], ['OK'])
        yield self.assertExchange(['AT+CSMS=1'], ['OK'])

    def make_service(self, command, cmdOptions=[], connOptions=[]):
        args = ['--device', '/dev/foo']
        args.extend(connOptions)
        args.extend([command] + cmdOptions)
        service_options = Options()
        service_options.parseOptions(args)
        service_maker = TxGSMServiceMaker()
        service = service_maker.makeService(service_options)
        service.startService()
        return service

    @inlineCallbacks
    def test_send_sms_command(self):
        service = self.make_service('send-sms', [
            '--to-addr', '+27761234567',
            '--message', 'hello world',
        ])
        yield self.assert_configure_modem()
        yield self.assertCommands(['AT+CMGS=23'])
        self.reply('> ', delimiter='')
        [pdu_payload] = yield self.wait_for_next_commands()
        self.reply('OK')
        response = yield service.onProtocol
        self.assertEqual(response, [
            {
                'command': ['AT+CMGS=23'],
                'expect': '> ',
                'response': ['> ']
            },
            {
                'command': ['0001000B917267214365F700000B'
                            'E8329BFD06DDDF723619\x1a'],
                'expect': 'OK',
                'response': ['OK']
            }
        ])

    @inlineCallbacks
    def test_list_sms(self):
        service = self.make_service('list-sms', [
            '--status', 4
        ])
        yield self.assert_configure_modem()
        with LogCatcher() as catcher:
            yield self.assertExchange(
                input=['AT+CMGL=4'],
                output=[
                    '+CMGL: 1,0,,39',
                    ('07911326040011F5240B911326880736F40000111081017362' +
                     '401654747A0E4ACF41F4329E0E6A97E7F3F0B90C8A01'),
                    '+CMGL: 2,0,,39',
                    ('07911326040011F5240B911326880736F40000111081017323' +
                     '401654747A0E4ACF41F4329E0E6A97E7F3F0B90C9201'),
                    'OK'
                ])
            yield service.onProtocol
        [sms1_log, sms2_log] = catcher.logs
        self.assertEqual('This is text message 1',
                         sms1_log['message'][0]['text'])
        self.assertEqual('This is text message 2',
                         sms2_log['message'][0]['text'])

    @inlineCallbacks
    def test_ussd_session(self):
        responses = []
        self.patch(USSDConsole, 'handle_response', responses.append)

        service = self.make_service('ussd-session', [
            '--code', '*100#'
        ])
        yield self.assert_configure_modem()
        yield self.assertExchange(
            input=['AT+CUSD=1,"*100#",15'],
            output=[
                'OK',
                ('+CUSD: 2,"Your balance is R48.70. Out of Airtime? '
                 'Dial *111# for Airtime Advance. T&Cs apply.",255')
            ])
        standard_io = yield service.onProtocol
        standard_io.loseConnection()
        [ussd_resp] = responses
        self.assertEqual(ussd_resp['response'], [
            'OK',
            ('+CUSD: 2,"Your balance is R48.70. Out of Airtime? '
             'Dial *111# for Airtime Advance. T&Cs apply.",255')
        ])

    def test_service_options(self):
        baudrate = 115200
        timeout = 1000
        service = self.make_service('probe-modem', connOptions=[
            '--baudrate', baudrate, '--timeout', timeout])

        self.assertEqual(service.conn_options['baudrate'], baudrate)
        self.assertEqual(service.conn_options['timeout'], timeout)

    @inlineCallbacks
    def test_probe_modem(self):
        service = self.make_service('probe-modem', [])
        with LogCatcher() as catcher:
            yield self.assertExchange(['ATE0'], ['OK'])
            yield self.assertExchange(['AT+CIMI'], ['01234123412341234', 'OK'])
            yield self.assertExchange(['AT+CGMM'], ['Foo Bar Corp', 'OK'])
            yield service.onProtocol

        [entry1, entry2] = catcher.logs
        self.assertEqual(entry1['message'][0], 'Manufacturer: Foo Bar Corp')
        self.assertEqual(entry2['message'][0], 'IMSI: 01234123412341234')
